@@ -1,32 +1,33 @@
-"""Config flow to configure the Elgato Key Light integration."""
+"""Config flow to configure the Elgato Light integration."""
 from __future__ import annotations
 
-from typing import Any, Dict
+from typing import Any
 
 from elgato import Elgato, ElgatoError
 import voluptuous as vol
 
-from homeassistant.config_entries import CONN_CLASS_LOCAL_POLL, ConfigFlow
+from homeassistant.components import zeroconf
+from homeassistant.config_entries import ConfigFlow
 from homeassistant.const import CONF_HOST, CONF_PORT
 from homeassistant.core import callback
+from homeassistant.data_entry_flow import FlowResult
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
 
-from .const import CONF_SERIAL_NUMBER, DOMAIN  # pylint: disable=unused-import
+from .const import CONF_SERIAL_NUMBER, DOMAIN
 
 
 class ElgatoFlowHandler(ConfigFlow, domain=DOMAIN):
-    """Handle a Elgato Key Light config flow."""
+    """Handle a Elgato Light config flow."""
 
     VERSION = 1
-    CONNECTION_CLASS = CONN_CLASS_LOCAL_POLL
 
     host: str
     port: int
     serial_number: str
 
     async def async_step_user(
-        self, user_input: Dict[str, Any] | None = None
-    ) -> Dict[str, Any]:
+        self, user_input: dict[str, Any] | None = None
+    ) -> FlowResult:
         """Handle a flow initiated by the user."""
         if user_input is None:
             return self._async_show_setup_form()
@@ -42,32 +43,33 @@ class ElgatoFlowHandler(ConfigFlow, domain=DOMAIN):
         return self._async_create_entry()
 
     async def async_step_zeroconf(
-        self, discovery_info: Dict[str, Any]
-    ) -> Dict[str, Any]:
+        self, discovery_info: zeroconf.ZeroconfServiceInfo
+    ) -> FlowResult:
         """Handle zeroconf discovery."""
-        self.host = discovery_info[CONF_HOST]
-        self.port = discovery_info[CONF_PORT]
+        self.host = discovery_info.host
+        self.port = discovery_info.port or 9123
 
         try:
             await self._get_elgato_serial_number()
         except ElgatoError:
             return self.async_abort(reason="cannot_connect")
 
+        self._set_confirm_only()
         return self.async_show_form(
             step_id="zeroconf_confirm",
             description_placeholders={"serial_number": self.serial_number},
         )
 
     async def async_step_zeroconf_confirm(
-        self, _: Dict[str, Any] | None = None
-    ) -> Dict[str, Any]:
+        self, _: dict[str, Any] | None = None
+    ) -> FlowResult:
         """Handle a flow initiated by zeroconf."""
         return self._async_create_entry()
 
     @callback
     def _async_show_setup_form(
-        self, errors: Dict[str, str] | None = None
-    ) -> Dict[str, Any]:
+        self, errors: dict[str, str] | None = None
+    ) -> FlowResult:
         """Show the setup form to the user."""
         return self.async_show_form(
             step_id="user",
@@ -81,7 +83,7 @@ class ElgatoFlowHandler(ConfigFlow, domain=DOMAIN):
         )
 
     @callback
-    def _async_create_entry(self) -> Dict[str, Any]:
+    def _async_create_entry(self) -> FlowResult:
         return self.async_create_entry(
             title=self.serial_number,
             data={
@@ -92,7 +94,7 @@ class ElgatoFlowHandler(ConfigFlow, domain=DOMAIN):
         )
 
     async def _get_elgato_serial_number(self, raise_on_progress: bool = True) -> None:
-        """Get device information from an Elgato Key Light device."""
+        """Get device information from an Elgato Light device."""
         session = async_get_clientsession(self.hass)
         elgato = Elgato(
             host=self.host,
