@@ -1,15 +1,21 @@
 """Support for GTFS (Google/General Transport Format Schema)."""
+from __future__ import annotations
+
+from collections.abc import Callable
 import datetime
 import logging
 import os
 import threading
-from typing import Any, Callable, Optional
+from typing import Any
 
 import pygtfs
 from sqlalchemy.sql import text
 import voluptuous as vol
 
-from homeassistant.components.sensor import PLATFORM_SCHEMA
+from homeassistant.components.sensor import (
+    PLATFORM_SCHEMA as SENSOR_PLATFORM_SCHEMA,
+    SensorEntity,
+)
 from homeassistant.const import (
     ATTR_ATTRIBUTION,
     CONF_NAME,
@@ -17,13 +23,9 @@ from homeassistant.const import (
     DEVICE_CLASS_TIMESTAMP,
     STATE_UNKNOWN,
 )
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.entity import Entity
-from homeassistant.helpers.typing import (
-    ConfigType,
-    DiscoveryInfoType,
-    HomeAssistantType,
-)
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 from homeassistant.util import slugify
 import homeassistant.util.dt as dt_util
 
@@ -76,6 +78,80 @@ ICONS = {
     5: "mdi:train-variant",
     6: "mdi:gondola",
     7: "mdi:stairs",
+    100: "mdi:train",
+    101: "mdi:train",
+    102: "mdi:train",
+    103: "mdi:train",
+    104: "mdi:train-car",
+    105: "mdi:train",
+    106: "mdi:train",
+    107: "mdi:train",
+    108: "mdi:train",
+    109: "mdi:train",
+    110: "mdi:train-variant",
+    111: "mdi:train-variant",
+    112: "mdi:train-variant",
+    113: "mdi:train-variant",
+    114: "mdi:train-variant",
+    115: "mdi:train-variant",
+    116: "mdi:train-variant",
+    117: "mdi:train-variant",
+    200: "mdi:bus",
+    201: "mdi:bus",
+    202: "mdi:bus",
+    203: "mdi:bus",
+    204: "mdi:bus",
+    205: "mdi:bus",
+    206: "mdi:bus",
+    207: "mdi:bus",
+    208: "mdi:bus",
+    209: "mdi:bus",
+    400: "mdi:subway-variant",
+    401: "mdi:subway-variant",
+    402: "mdi:subway",
+    403: "mdi:subway-variant",
+    404: "mdi:subway-variant",
+    405: "mdi:subway-variant",
+    700: "mdi:bus",
+    701: "mdi:bus",
+    702: "mdi:bus",
+    703: "mdi:bus",
+    704: "mdi:bus",
+    705: "mdi:bus",
+    706: "mdi:bus",
+    707: "mdi:bus",
+    708: "mdi:bus",
+    709: "mdi:bus",
+    710: "mdi:bus",
+    711: "mdi:bus",
+    712: "mdi:bus-school",
+    713: "mdi:bus-school",
+    714: "mdi:bus",
+    715: "mdi:bus",
+    716: "mdi:bus",
+    800: "mdi:bus",
+    900: "mdi:tram",
+    901: "mdi:tram",
+    902: "mdi:tram",
+    903: "mdi:tram",
+    904: "mdi:tram",
+    905: "mdi:tram",
+    906: "mdi:tram",
+    1000: "mdi:ferry",
+    1100: "mdi:airplane",
+    1200: "mdi:ferry",
+    1300: "mdi:airplane",
+    1400: "mdi:gondola",
+    1500: "mdi:taxi",
+    1501: "mdi:taxi",
+    1502: "mdi:ferry",
+    1503: "mdi:train-variant",
+    1504: "mdi:bicycle-basket",
+    1505: "mdi:taxi",
+    1506: "mdi:car-multiple",
+    1507: "mdi:taxi",
+    1700: "mdi:train-car",
+    1702: "mdi:horse-variant",
 }
 LOCATION_TYPE_DEFAULT = "Stop"
 LOCATION_TYPE_OPTIONS = {
@@ -100,6 +176,80 @@ ROUTE_TYPE_OPTIONS = {
     5: "Cable Tram",
     6: "Aerial Lift",
     7: "Funicular",
+    100: "Railway Service",
+    101: "High Speed Rail Service",
+    102: "Long Distance Trains",
+    103: "Inter Regional Rail Service",
+    104: "Car Transport Rail Service",
+    105: "Sleeper Rail Service",
+    106: "Regional Rail Service",
+    107: "Tourist Railway Service",
+    108: "Rail Shuttle (Within Complex)",
+    109: "Suburban Railway",
+    110: "Replacement Rail Service",
+    111: "Special Rail Service",
+    112: "Lorry Transport Rail Service",
+    113: "All Rail Services",
+    114: "Cross-Country Rail Service",
+    115: "Vehicle Transport Rail Service",
+    116: "Rack and Pinion Railway",
+    117: "Additional Rail Service",
+    200: "Coach Service",
+    201: "International Coach Service",
+    202: "National Coach Service",
+    203: "Shuttle Coach Service",
+    204: "Regional Coach Service",
+    205: "Special Coach Service",
+    206: "Sightseeing Coach Service",
+    207: "Tourist Coach Service",
+    208: "Commuter Coach Service",
+    209: "All Coach Services",
+    400: "Urban Railway Service",
+    401: "Metro Service",
+    402: "Underground Service",
+    403: "Urban Railway Service",
+    404: "All Urban Railway Services",
+    405: "Monorail",
+    700: "Bus Service",
+    701: "Regional Bus Service",
+    702: "Express Bus Service",
+    703: "Stopping Bus Service",
+    704: "Local Bus Service",
+    705: "Night Bus Service",
+    706: "Post Bus Service",
+    707: "Special Needs Bus",
+    708: "Mobility Bus Service",
+    709: "Mobility Bus for Registered Disabled",
+    710: "Sightseeing Bus",
+    711: "Shuttle Bus",
+    712: "School Bus",
+    713: "School and Public Service Bus",
+    714: "Rail Replacement Bus Service",
+    715: "Demand and Response Bus Service",
+    716: "All Bus Services",
+    800: "Trolleybus Service",
+    900: "Tram Service",
+    901: "City Tram Service",
+    902: "Local Tram Service",
+    903: "Regional Tram Service",
+    904: "Sightseeing Tram Service",
+    905: "Shuttle Tram Service",
+    906: "All Tram Services",
+    1000: "Water Transport Service",
+    1100: "Air Service",
+    1200: "Ferry Service",
+    1300: "Aerial Lift Service",
+    1400: "Funicular Service",
+    1500: "Taxi Service",
+    1501: "Communal Taxi Service",
+    1502: "Water Taxi Service",
+    1503: "Rail Taxi Service",
+    1504: "Bike Taxi Service",
+    1505: "Licensed Taxi Service",
+    1506: "Private Hire Service Vehicle",
+    1507: "All Taxi Services",
+    1700: "Miscellaneous Service",
+    1702: "Horse-drawn Carriage",
 }
 TIMEPOINT_DEFAULT = True
 TIMEPOINT_OPTIONS = {0: False, 1: True}
@@ -108,8 +258,8 @@ WHEELCHAIR_ACCESS_OPTIONS = {1: True, 2: False}
 WHEELCHAIR_BOARDING_DEFAULT = STATE_UNKNOWN
 WHEELCHAIR_BOARDING_OPTIONS = {1: True, 2: False}
 
-PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
-    {  # type: ignore
+PLATFORM_SCHEMA = SENSOR_PLATFORM_SCHEMA.extend(
+    {
         vol.Required(CONF_ORIGIN): cv.string,
         vol.Required(CONF_DESTINATION): cv.string,
         vol.Required(CONF_DATA): cv.string,
@@ -333,10 +483,10 @@ def get_next_departure(
 
 
 def setup_platform(
-    hass: HomeAssistantType,
+    hass: HomeAssistant,
     config: ConfigType,
     add_entities: Callable[[list], None],
-    discovery_info: Optional[DiscoveryInfoType] = None,
+    discovery_info: DiscoveryInfoType | None = None,
 ) -> None:
     """Set up the GTFS sensor."""
     gtfs_dir = hass.config.path(DEFAULT_PATH)
@@ -344,7 +494,7 @@ def setup_platform(
     origin = config.get(CONF_ORIGIN)
     destination = config.get(CONF_DESTINATION)
     name = config.get(CONF_NAME)
-    offset = config.get(CONF_OFFSET)
+    offset: datetime.timedelta = config[CONF_OFFSET]
     include_tomorrow = config[CONF_TOMORROW]
 
     if not os.path.exists(gtfs_dir):
@@ -369,16 +519,18 @@ def setup_platform(
     )
 
 
-class GTFSDepartureSensor(Entity):
+class GTFSDepartureSensor(SensorEntity):
     """Implementation of a GTFS departure sensor."""
 
+    _attr_device_class = DEVICE_CLASS_TIMESTAMP
+
     def __init__(
         self,
         gtfs: Any,
-        name: Optional[Any],
+        name: Any | None,
         origin: Any,
         destination: Any,
-        offset: cv.time_period,
+        offset: datetime.timedelta,
         include_tomorrow: bool,
     ) -> None:
         """Initialize the sensor."""
@@ -392,11 +544,11 @@ class GTFSDepartureSensor(Entity):
         self._available = False
         self._icon = ICON
         self._name = ""
-        self._state: Optional[str] = None
-        self._attributes = {}
+        self._state: datetime.datetime | None = None
+        self._attributes: dict[str, Any] = {}
 
         self._agency = None
-        self._departure = {}
+        self._departure: dict[str, Any] = {}
         self._destination = None
         self._origin = None
         self._route = None
@@ -411,7 +563,7 @@ class GTFSDepartureSensor(Entity):
         return self._name
 
     @property
-    def state(self) -> Optional[str]:  # type: ignore
+    def native_value(self) -> datetime.datetime | None:
         """Return the state of the sensor."""
         return self._state
 
@@ -421,7 +573,7 @@ class GTFSDepartureSensor(Entity):
         return self._available
 
     @property
-    def device_state_attributes(self) -> dict:
+    def extra_state_attributes(self) -> dict:
         """Return the state attributes."""
         return self._attributes
 
@@ -430,11 +582,6 @@ class GTFSDepartureSensor(Entity):
         """Icon to use in the frontend, if any."""
         return self._icon
 
-    @property
-    def device_class(self) -> str:
-        """Return the class of this device."""
-        return DEVICE_CLASS_TIMESTAMP
-
     def update(self) -> None:
         """Get the latest data from GTFS and update the states."""
         with self.lock:
@@ -472,9 +619,7 @@ class GTFSDepartureSensor(Entity):
             if not self._departure:
                 self._state = None
             else:
-                self._state = dt_util.as_utc(
-                    self._departure["departure_time"]
-                ).isoformat()
+                self._state = self._departure["departure_time"]
 
             # Fetch trip and route details once, unless updated
             if not self._departure:
@@ -550,7 +695,7 @@ class GTFSDepartureSensor(Entity):
                 del self._attributes[ATTR_LAST]
 
         # Add contextual information
-        self._attributes[ATTR_OFFSET] = self._offset.seconds / 60
+        self._attributes[ATTR_OFFSET] = self._offset.total_seconds() / 60
 
         if self._state is None:
             self._attributes[ATTR_INFO] = (
@@ -659,11 +804,11 @@ class GTFSDepartureSensor(Entity):
     @staticmethod
     def dict_for_table(resource: Any) -> dict:
         """Return a dictionary for the SQLAlchemy resource given."""
-        return dict(
-            (col, getattr(resource, col)) for col in resource.__table__.columns.keys()
-        )
+        return {
+            col: getattr(resource, col) for col in resource.__table__.columns.keys()
+        }
 
-    def append_keys(self, resource: dict, prefix: Optional[str] = None) -> None:
+    def append_keys(self, resource: dict, prefix: str | None = None) -> None:
         """Properly format key val pairs to append to attributes."""
         for attr, val in resource.items():
             if val == "" or val is None or attr == "feed_id":
