@@ -27,7 +27,7 @@ from homeassistant.const import (
     CONF_TYPE,
     TEMP_CELSIUS,
 )
-from homeassistant.core import HomeAssistant, split_entity_id
+from homeassistant.core import HomeAssistant, callback, split_entity_id
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.storage import STORAGE_DIR
 import homeassistant.util.temperature as temp_util
@@ -294,9 +294,7 @@ def get_media_player_features(state):
 
 def validate_media_player_features(state, feature_list):
     """Validate features for media players."""
-    supported_modes = get_media_player_features(state)
-
-    if not supported_modes:
+    if not (supported_modes := get_media_player_features(state)):
         _LOGGER.error("%s does not support any media_player features", state.entity_id)
         return False
 
@@ -317,7 +315,7 @@ def validate_media_player_features(state, feature_list):
     return True
 
 
-def show_setup_message(hass, entry_id, bridge_name, pincode, uri):
+def async_show_setup_message(hass, entry_id, bridge_name, pincode, uri):
     """Display persistent notification with setup information."""
     pin = pincode.decode()
     _LOGGER.info("Pincode: %s", pin)
@@ -336,12 +334,14 @@ def show_setup_message(hass, entry_id, bridge_name, pincode, uri):
         f"### {pin}\n"
         f"![image](/api/homekit/pairingqr?{entry_id}-{pairing_secret})"
     )
-    hass.components.persistent_notification.create(message, "HomeKit Pairing", entry_id)
+    hass.components.persistent_notification.async_create(
+        message, "HomeKit Pairing", entry_id
+    )
 
 
-def dismiss_setup_message(hass, entry_id):
+def async_dismiss_setup_message(hass, entry_id):
     """Dismiss persistent notification and remove QR code."""
-    hass.components.persistent_notification.dismiss(entry_id)
+    hass.components.persistent_notification.async_dismiss(entry_id)
 
 
 def convert_to_float(state):
@@ -433,34 +433,32 @@ def _get_test_socket():
     return test_socket
 
 
-def port_is_available(port: int) -> bool:
+@callback
+def async_port_is_available(port: int) -> bool:
     """Check to see if a port is available."""
-    test_socket = _get_test_socket()
     try:
-        test_socket.bind(("", port))
+        _get_test_socket().bind(("", port))
     except OSError:
         return False
-
     return True
 
 
-async def async_find_next_available_port(hass: HomeAssistant, start_port: int) -> int:
+@callback
+def async_find_next_available_port(hass: HomeAssistant, start_port: int) -> int:
     """Find the next available port not assigned to a config entry."""
     exclude_ports = {
         entry.data[CONF_PORT]
         for entry in hass.config_entries.async_entries(DOMAIN)
         if CONF_PORT in entry.data
     }
-
-    return await hass.async_add_executor_job(
-        _find_next_available_port, start_port, exclude_ports
-    )
+    return _async_find_next_available_port(start_port, exclude_ports)
 
 
-def _find_next_available_port(start_port: int, exclude_ports: set) -> int:
+@callback
+def _async_find_next_available_port(start_port: int, exclude_ports: set) -> int:
     """Find the next available port starting with the given port."""
     test_socket = _get_test_socket()
-    for port in range(start_port, MAX_PORT):
+    for port in range(start_port, MAX_PORT + 1):
         if port in exclude_ports:
             continue
         try:
