@@ -10,42 +10,55 @@ from xknx import XKNX
 from xknx.core import XknxConnectionState
 from xknx.core.telegram_queue import TelegramQueue
 from xknx.dpt import DPTArray, DPTBase, DPTBinary
-from xknx.exceptions import XKNXException
+from xknx.exceptions import ConversionError, XKNXException
 from xknx.io import ConnectionConfig, ConnectionType
 from xknx.telegram import AddressFilter, Telegram
-from xknx.telegram.address import parse_device_group_address
+from xknx.telegram.address import (
+    DeviceGroupAddress,
+    GroupAddress,
+    InternalGroupAddress,
+    parse_device_group_address,
+)
 from xknx.telegram.apci import GroupValueRead, GroupValueResponse, GroupValueWrite
 
+from homeassistant import config_entries
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
+    CONF_EVENT,
     CONF_HOST,
     CONF_PORT,
+    CONF_TYPE,
     EVENT_HOMEASSISTANT_STOP,
-    SERVICE_RELOAD,
+    Platform,
 )
 from homeassistant.core import Event, HomeAssistant, ServiceCall
-from homeassistant.exceptions import HomeAssistantError
+from homeassistant.exceptions import ConfigEntryNotReady, HomeAssistantError
 from homeassistant.helpers import discovery
 import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.entity_platform import async_get_platforms
 from homeassistant.helpers.reload import async_integration_yaml_config
 from homeassistant.helpers.service import async_register_admin_service
 from homeassistant.helpers.typing import ConfigType
 
 from .const import (
+    CONF_KNX_CONNECTION_TYPE,
     CONF_KNX_EXPOSE,
     CONF_KNX_INDIVIDUAL_ADDRESS,
     CONF_KNX_ROUTING,
     CONF_KNX_TUNNELING,
+    DATA_HASS_CONFIG,
+    DATA_KNX_CONFIG,
     DOMAIN,
     KNX_ADDRESS,
-    SupportedPlatforms,
+    SUPPORTED_PLATFORMS,
 )
 from .expose import KNXExposeSensor, KNXExposeTime, create_knx_exposure
 from .schema import (
     BinarySensorSchema,
+    ButtonSchema,
     ClimateSchema,
     ConnectionSchema,
     CoverSchema,
+    EventSchema,
     ExposeSchema,
     FanSchema,
     LightSchema,
@@ -77,6 +90,15 @@ SERVICE_KNX_READ: Final = "read"
 CONFIG_SCHEMA = vol.Schema(
     {
         DOMAIN: vol.All(
+            # deprecated since 2021.12
+            cv.deprecated(ConnectionSchema.CONF_KNX_STATE_UPDATER),
+            cv.deprecated(ConnectionSchema.CONF_KNX_RATE_LIMIT),
+            cv.deprecated(CONF_KNX_ROUTING),
+            cv.deprecated(CONF_KNX_TUNNELING),
+            cv.deprecated(CONF_KNX_INDIVIDUAL_ADDRESS),
+            cv.deprecated(ConnectionSchema.CONF_KNX_MCAST_GRP),
+            cv.deprecated(ConnectionSchema.CONF_KNX_MCAST_PORT),
+            cv.deprecated(CONF_KNX_EVENT_FILTER),
             # deprecated since 2021.4
             cv.deprecated("config_file"),
             # deprecated since 2021.2
@@ -89,8 +111,10 @@ CONFIG_SCHEMA = vol.Schema(
                     vol.Optional(CONF_KNX_EVENT_FILTER, default=[]): vol.All(
                         cv.ensure_list, [cv.string]
                     ),
+                    **EventSchema.SCHEMA,
                     **ExposeSchema.platform_node(),
                     **BinarySensorSchema.platform_node(),
+                    **ButtonSchema.platform_node(),
                     **ClimateSchema.platform_node(),
                     **CoverSchema.platform_node(),
                     **FanSchema.platform_node(),
@@ -149,6 +173,7 @@ SERVICE_KNX_EVENT_REGISTER_SCHEMA = vol.Schema(
             cv.ensure_list,
             [ga_validator],
         ),
+        vol.Optional(CONF_TYPE): sensor_type_validator,
         vol.Optional(SERVICE_KNX_ATTR_REMOVE, default=False): cv.boolean,
     }
 )
@@ -171,35 +196,77 @@ SERVICE_KNX_EXPOSURE_REGISTER_SCHEMA = vol.Any(
 
 
 async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
-    """Set up the KNX integration."""
-    try:
-        knx_module = KNXModule(hass, config)
-        hass.data[DOMAIN] = knx_module
-        await knx_module.start()
-    except XKNXException as ex:
-        _LOGGER.warning("Could not connect to KNX interface: %s", ex)
-        hass.components.persistent_notification.async_create(
-            f"Could not connect to KNX interface: <br><b>{ex}</b>", title="KNX"
+    """Start the KNX integration."""
+    hass.data[DATA_HASS_CONFIG] = config
+    conf: ConfigType | None = config.get(DOMAIN)
+
+    if conf is None:
+        # If we have a config entry, setup is done by that config entry.
+        # If there is no config entry, this should fail.
+        return bool(hass.config_entries.async_entries(DOMAIN))
+
+    conf = dict(conf)
+
+    hass.data[DATA_KNX_CONFIG] = conf
+
+    # Only import if we haven't before.
+    if not hass.config_entries.async_entries(DOMAIN):
+        hass.async_create_task(
+            hass.config_entries.flow.async_init(
+                DOMAIN, context={"source": config_entries.SOURCE_IMPORT}, data=conf
+            )
         )
 
-    if CONF_KNX_EXPOSE in config[DOMAIN]:
-        for expose_config in config[DOMAIN][CONF_KNX_EXPOSE]:
+    return True
+
+
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
+    """Load a config entry."""
+    conf = hass.data.get(DATA_KNX_CONFIG)
+
+    #  When reloading
+    if conf is None:
+        conf = await async_integration_yaml_config(hass, DOMAIN)
+        if not conf or DOMAIN not in conf:
+            return False
+
+        conf = conf[DOMAIN]
+
+    # If user didn't have configuration.yaml config, generate defaults
+    if conf is None:
+        conf = CONFIG_SCHEMA({DOMAIN: dict(entry.data)})[DOMAIN]
+
+    config = {**conf, **entry.data}
+
+    try:
+        knx_module = KNXModule(hass, config, entry)
+        await knx_module.start()
+    except XKNXException as ex:
+        raise ConfigEntryNotReady from ex
+
+    hass.data[DATA_KNX_CONFIG] = conf
+    hass.data[DOMAIN] = knx_module
+
+    if CONF_KNX_EXPOSE in config:
+        for expose_config in config[CONF_KNX_EXPOSE]:
             knx_module.exposures.append(
                 create_knx_exposure(hass, knx_module.xknx, expose_config)
             )
 
-    for platform in SupportedPlatforms:
-        if platform.value not in config[DOMAIN]:
-            continue
+    hass.config_entries.async_setup_platforms(
+        entry,
+        [
+            platform
+            for platform in SUPPORTED_PLATFORMS
+            if platform in config and platform is not Platform.NOTIFY
+        ],
+    )
+
+    # set up notify platform, no entry support for notify component yet
+    if NotifySchema.PLATFORM in config:
         hass.async_create_task(
             discovery.async_load_platform(
-                hass,
-                platform.value,
-                DOMAIN,
-                {
-                    "platform_config": config[DOMAIN][platform.value],
-                },
-                config,
+                hass, "notify", DOMAIN, {}, hass.data[DATA_HASS_CONFIG]
             )
         )
 
@@ -233,125 +300,111 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
         schema=SERVICE_KNX_EXPOSURE_REGISTER_SCHEMA,
     )
 
-    async def reload_service_handler(service_call: ServiceCall) -> None:
-        """Remove all KNX components and load new ones from config."""
-
-        # First check for config file. If for some reason it is no longer there
-        # or knx is no longer mentioned, stop the reload.
-        config = await async_integration_yaml_config(hass, DOMAIN)
-        if not config or DOMAIN not in config:
-            return
-
-        await asyncio.gather(
-            *(platform.async_reset() for platform in async_get_platforms(hass, DOMAIN))
-        )
-        await knx_module.xknx.stop()
-
-        await async_setup(hass, config)
-
-    async_register_admin_service(
-        hass, DOMAIN, SERVICE_RELOAD, reload_service_handler, schema=vol.Schema({})
-    )
-
     return True
 
 
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
+    """Unloading the KNX platforms."""
+    #  if not loaded directly return
+    if not hass.data.get(DOMAIN):
+        return True
+
+    knx_module: KNXModule = hass.data[DOMAIN]
+    for exposure in knx_module.exposures:
+        exposure.shutdown()
+
+    unload_ok = await hass.config_entries.async_unload_platforms(
+        entry,
+        [
+            platform
+            for platform in SUPPORTED_PLATFORMS
+            if platform in hass.data[DATA_KNX_CONFIG]
+            and platform is not Platform.NOTIFY
+        ],
+    )
+    if unload_ok:
+        await knx_module.stop()
+        hass.data.pop(DOMAIN)
+        hass.data.pop(DATA_KNX_CONFIG)
+
+    return unload_ok
+
+
+async def async_update_entry(hass: HomeAssistant, entry: ConfigEntry) -> None:
+    """Update a given config entry."""
+    await hass.config_entries.async_reload(entry.entry_id)
+
+
 class KNXModule:
     """Representation of KNX Object."""
 
-    def __init__(self, hass: HomeAssistant, config: ConfigType) -> None:
+    def __init__(
+        self, hass: HomeAssistant, config: ConfigType, entry: ConfigEntry
+    ) -> None:
         """Initialize KNX module."""
         self.hass = hass
         self.config = config
         self.connected = False
         self.exposures: list[KNXExposeSensor | KNXExposeTime] = []
         self.service_exposures: dict[str, KNXExposeSensor | KNXExposeTime] = {}
+        self.entry = entry
 
         self.init_xknx()
-        self._knx_event_callback: TelegramQueue.Callback = self.register_callback()
         self.xknx.connection_manager.register_connection_state_changed_cb(
             self.connection_state_changed_cb
         )
 
+        self._address_filter_transcoder: dict[AddressFilter, type[DPTBase]] = {}
+        self._group_address_transcoder: dict[DeviceGroupAddress, type[DPTBase]] = {}
+        self._knx_event_callback: TelegramQueue.Callback = (
+            self.register_event_callback()
+        )
+
+        self.entry.async_on_unload(
+            self.hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, self.stop)
+        )
+
+        self.entry.async_on_unload(self.entry.add_update_listener(async_update_entry))
+
     def init_xknx(self) -> None:
         """Initialize XKNX object."""
         self.xknx = XKNX(
-            own_address=self.config[DOMAIN][CONF_KNX_INDIVIDUAL_ADDRESS],
-            rate_limit=self.config[DOMAIN][ConnectionSchema.CONF_KNX_RATE_LIMIT],
-            multicast_group=self.config[DOMAIN][ConnectionSchema.CONF_KNX_MCAST_GRP],
-            multicast_port=self.config[DOMAIN][ConnectionSchema.CONF_KNX_MCAST_PORT],
+            own_address=self.config[CONF_KNX_INDIVIDUAL_ADDRESS],
+            rate_limit=self.config[ConnectionSchema.CONF_KNX_RATE_LIMIT],
+            multicast_group=self.config[ConnectionSchema.CONF_KNX_MCAST_GRP],
+            multicast_port=self.config[ConnectionSchema.CONF_KNX_MCAST_PORT],
             connection_config=self.connection_config(),
-            state_updater=self.config[DOMAIN][ConnectionSchema.CONF_KNX_STATE_UPDATER],
+            state_updater=self.config[ConnectionSchema.CONF_KNX_STATE_UPDATER],
         )
 
     async def start(self) -> None:
         """Start XKNX object. Connect to tunneling or Routing device."""
         await self.xknx.start()
-        self.hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, self.stop)
 
-    async def stop(self, event: Event) -> None:
+    async def stop(self, event: Event | None = None) -> None:
         """Stop XKNX object. Disconnect from tunneling or Routing device."""
         await self.xknx.stop()
 
     def connection_config(self) -> ConnectionConfig:
         """Return the connection_config."""
-        if CONF_KNX_TUNNELING in self.config[DOMAIN]:
-            return self.connection_config_tunneling()
-        if CONF_KNX_ROUTING in self.config[DOMAIN]:
-            return self.connection_config_routing()
-        return ConnectionConfig(auto_reconnect=True)
-
-    def connection_config_routing(self) -> ConnectionConfig:
-        """Return the connection_config if routing is configured."""
-        local_ip = None
-        # all configuration values are optional
-        if self.config[DOMAIN][CONF_KNX_ROUTING] is not None:
-            local_ip = self.config[DOMAIN][CONF_KNX_ROUTING].get(
-                ConnectionSchema.CONF_KNX_LOCAL_IP
+        _conn_type: str = self.config[CONF_KNX_CONNECTION_TYPE]
+        if _conn_type == CONF_KNX_ROUTING:
+            return ConnectionConfig(
+                connection_type=ConnectionType.ROUTING,
+                local_ip=self.config.get(ConnectionSchema.CONF_KNX_LOCAL_IP),
+                auto_reconnect=True,
+            )
+        if _conn_type == CONF_KNX_TUNNELING:
+            return ConnectionConfig(
+                connection_type=ConnectionType.TUNNELING,
+                gateway_ip=self.config[CONF_HOST],
+                gateway_port=self.config[CONF_PORT],
+                local_ip=self.config.get(ConnectionSchema.CONF_KNX_LOCAL_IP),
+                route_back=self.config.get(ConnectionSchema.CONF_KNX_ROUTE_BACK, False),
+                auto_reconnect=True,
             )
-        return ConnectionConfig(
-            connection_type=ConnectionType.ROUTING, local_ip=local_ip
-        )
 
-    def connection_config_tunneling(self) -> ConnectionConfig:
-        """Return the connection_config if tunneling is configured."""
-        gateway_ip = self.config[DOMAIN][CONF_KNX_TUNNELING][CONF_HOST]
-        gateway_port = self.config[DOMAIN][CONF_KNX_TUNNELING][CONF_PORT]
-        local_ip = self.config[DOMAIN][CONF_KNX_TUNNELING].get(
-            ConnectionSchema.CONF_KNX_LOCAL_IP
-        )
-        route_back = self.config[DOMAIN][CONF_KNX_TUNNELING][
-            ConnectionSchema.CONF_KNX_ROUTE_BACK
-        ]
-        return ConnectionConfig(
-            connection_type=ConnectionType.TUNNELING,
-            gateway_ip=gateway_ip,
-            gateway_port=gateway_port,
-            local_ip=local_ip,
-            route_back=route_back,
-            auto_reconnect=True,
-        )
-
-    async def telegram_received_cb(self, telegram: Telegram) -> None:
-        """Call invoked after a KNX telegram was received."""
-        data = None
-        # Not all telegrams have serializable data.
-        if (
-            isinstance(telegram.payload, (GroupValueWrite, GroupValueResponse))
-            and telegram.payload.value is not None
-        ):
-            data = telegram.payload.value.value
-
-        self.hass.bus.async_fire(
-            "knx_event",
-            {
-                "data": data,
-                "destination": str(telegram.destination_address),
-                "direction": telegram.direction.value,
-                "source": str(telegram.source_address),
-                "telegramtype": telegram.payload.__class__.__name__,
-            },
-        )
+        return ConnectionConfig(auto_reconnect=True)
 
     async def connection_state_changed_cb(self, state: XknxConnectionState) -> None:
         """Call invoked after a KNX connection state change was received."""
@@ -359,11 +412,69 @@ class KNXModule:
         if tasks := [device.after_update() for device in self.xknx.devices]:
             await asyncio.gather(*tasks)
 
-    def register_callback(self) -> TelegramQueue.Callback:
-        """Register callback within XKNX TelegramQueue."""
-        address_filters = list(
-            map(AddressFilter, self.config[DOMAIN][CONF_KNX_EVENT_FILTER])
+    async def telegram_received_cb(self, telegram: Telegram) -> None:
+        """Call invoked after a KNX telegram was received."""
+        # Not all telegrams have serializable data.
+        data: int | tuple[int, ...] | None = None
+        value = None
+        if (
+            isinstance(telegram.payload, (GroupValueWrite, GroupValueResponse))
+            and telegram.payload.value is not None
+            and isinstance(
+                telegram.destination_address, (GroupAddress, InternalGroupAddress)
+            )
+        ):
+            data = telegram.payload.value.value
+
+            if isinstance(data, tuple):
+                if transcoder := (
+                    self._group_address_transcoder.get(telegram.destination_address)
+                    or next(
+                        (
+                            _transcoder
+                            for _filter, _transcoder in self._address_filter_transcoder.items()
+                            if _filter.match(telegram.destination_address)
+                        ),
+                        None,
+                    )
+                ):
+                    try:
+                        value = transcoder.from_knx(data)
+                    except ConversionError as err:
+                        _LOGGER.warning(
+                            "Error in `knx_event` at decoding type '%s' from telegram %s\n%s",
+                            transcoder.__name__,
+                            telegram,
+                            err,
+                        )
+
+        self.hass.bus.async_fire(
+            "knx_event",
+            {
+                "data": data,
+                "destination": str(telegram.destination_address),
+                "direction": telegram.direction.value,
+                "value": value,
+                "source": str(telegram.source_address),
+                "telegramtype": telegram.payload.__class__.__name__,
+            },
         )
+
+    def register_event_callback(self) -> TelegramQueue.Callback:
+        """Register callback for knx_event within XKNX TelegramQueue."""
+        # backwards compatibility for deprecated CONF_KNX_EVENT_FILTER
+        # use `address_filters = []` when this is not needed anymore
+        address_filters = list(map(AddressFilter, self.config[CONF_KNX_EVENT_FILTER]))
+        for filter_set in self.config[CONF_EVENT]:
+            _filters = list(map(AddressFilter, filter_set[KNX_ADDRESS]))
+            address_filters.extend(_filters)
+            if (dpt := filter_set.get(CONF_TYPE)) and (
+                transcoder := DPTBase.parse_transcoder(dpt)
+            ):
+                self._address_filter_transcoder.update(
+                    {_filter: transcoder for _filter in _filters}  # type: ignore[misc]
+                )
+
         return self.xknx.telegram_queue.register_telegram_received_cb(
             self.telegram_received_cb,
             address_filters=address_filters,
@@ -374,7 +485,7 @@ class KNXModule:
     async def service_event_register_modify(self, call: ServiceCall) -> None:
         """Service for adding or removing a GroupAddress to the knx_event filter."""
         attr_address = call.data[KNX_ADDRESS]
-        group_addresses = map(parse_device_group_address, attr_address)
+        group_addresses = list(map(parse_device_group_address, attr_address))
 
         if call.data.get(SERVICE_KNX_ATTR_REMOVE):
             for group_address in group_addresses:
@@ -385,8 +496,16 @@ class KNXModule:
                         "Service event_register could not remove event for '%s'",
                         str(group_address),
                     )
+                if group_address in self._group_address_transcoder:
+                    del self._group_address_transcoder[group_address]
             return
 
+        if (dpt := call.data.get(CONF_TYPE)) and (
+            transcoder := DPTBase.parse_transcoder(dpt)
+        ):
+            self._group_address_transcoder.update(
+                {_address: transcoder for _address in group_addresses}  # type: ignore[misc]
+            )
         for group_address in group_addresses:
             if group_address in self._knx_event_callback.group_addresses:
                 continue
