@@ -6,14 +6,13 @@ import logging
 from typing import Any
 import urllib.parse
 
-from pysonos import alarms
-from pysonos.core import (
+from soco import alarms
+from soco.core import (
     MUSIC_SRC_LINE_IN,
     MUSIC_SRC_RADIO,
     PLAY_MODE_BY_MEANING,
     PLAY_MODES,
 )
-from pysonos.exceptions import SoCoUPnPException
 import voluptuous as vol
 
 from homeassistant.components.media_player import MediaPlayerEntity
@@ -54,6 +53,7 @@ from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.network import is_internal_request
 
 from .const import (
+    DATA_SONOS,
     DOMAIN as SONOS_DOMAIN,
     MEDIA_TYPES_TO_SONOS,
     PLAYABLE_MEDIA_TYPES,
@@ -108,7 +108,6 @@ SERVICE_RESTORE = "restore"
 SERVICE_SET_TIMER = "set_sleep_timer"
 SERVICE_CLEAR_TIMER = "clear_sleep_timer"
 SERVICE_UPDATE_ALARM = "update_alarm"
-SERVICE_SET_OPTION = "set_option"
 SERVICE_PLAY_QUEUE = "play_queue"
 SERVICE_REMOVE_FROM_QUEUE = "remove_from_queue"
 
@@ -119,11 +118,7 @@ ATTR_ENABLED = "enabled"
 ATTR_INCLUDE_LINKED_ZONES = "include_linked_zones"
 ATTR_MASTER = "master"
 ATTR_WITH_GROUP = "with_group"
-ATTR_BUTTONS_ENABLED = "buttons_enabled"
-ATTR_NIGHT_SOUND = "night_sound"
-ATTR_SPEECH_ENHANCE = "speech_enhance"
 ATTR_QUEUE_POSITION = "queue_position"
-ATTR_STATUS_LIGHT = "status_light"
 
 
 async def async_setup_entry(
@@ -137,6 +132,7 @@ async def async_setup_entry(
     @callback
     def async_create_entities(speaker: SonosSpeaker) -> None:
         """Handle device discovery and create entities."""
+        _LOGGER.debug("Creating media_player on %s", speaker.zone_name)
         async_add_entities([SonosMediaPlayerEntity(speaker)])
 
     @service.verify_domain_control(hass, SONOS_DOMAIN)
@@ -227,17 +223,6 @@ async def async_setup_entry(
         "set_alarm",
     )
 
-    platform.async_register_entity_service(  # type: ignore
-        SERVICE_SET_OPTION,
-        {
-            vol.Optional(ATTR_BUTTONS_ENABLED): cv.boolean,
-            vol.Optional(ATTR_NIGHT_SOUND): cv.boolean,
-            vol.Optional(ATTR_SPEECH_ENHANCE): cv.boolean,
-            vol.Optional(ATTR_STATUS_LIGHT): cv.boolean,
-        },
-        "set_option",
-    )
-
     platform.async_register_entity_service(  # type: ignore
         SERVICE_PLAY_QUEUE,
         {vol.Optional(ATTR_QUEUE_POSITION): cv.positive_int},
@@ -254,25 +239,24 @@ async def async_setup_entry(
 class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
     """Representation of a Sonos entity."""
 
+    _attr_supported_features = SUPPORT_SONOS
+    _attr_media_content_type = MEDIA_TYPE_MUSIC
+
+    def __init__(self, speaker: SonosSpeaker) -> None:
+        """Initialize the media player entity."""
+        super().__init__(speaker)
+        self._attr_unique_id = self.soco.uid
+        self._attr_name = self.speaker.zone_name
+
     @property
     def coordinator(self) -> SonosSpeaker:
         """Return the current coordinator SonosSpeaker."""
         return self.speaker.coordinator or self.speaker
 
-    @property
-    def unique_id(self) -> str:
-        """Return a unique ID."""
-        return self.soco.uid  # type: ignore[no-any-return]
-
     def __hash__(self) -> int:
         """Return a hash of self."""
         return hash(self.unique_id)
 
-    @property
-    def name(self) -> str:
-        """Return the name of the entity."""
-        return self.speaker.zone_name  # type: ignore[no-any-return]
-
     @property  # type: ignore[misc]
     def state(self) -> str:
         """Return the state of the entity."""
@@ -292,8 +276,11 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
             return STATE_PLAYING
         return STATE_IDLE
 
-    async def async_update(self) -> None:
+    async def _async_poll(self) -> None:
         """Retrieve latest state by polling."""
+        await self.hass.data[DATA_SONOS].favorites[
+            self.speaker.household_id
+        ].async_poll()
         await self.hass.async_add_executor_job(self._update)
 
     def _update(self) -> None:
@@ -335,11 +322,6 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
         """Content id of current playing media."""
         return self.media.uri
 
-    @property
-    def media_content_type(self) -> str:
-        """Content type of current playing media."""
-        return MEDIA_TYPE_MUSIC
-
     @property  # type: ignore[misc]
     def media_duration(self) -> float | None:
         """Duration of current playing media in seconds."""
@@ -365,6 +347,11 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
         """Channel currently playing."""
         return self.media.channel or None
 
+    @property
+    def media_playlist(self) -> str | None:
+        """Title of playlist currently playing."""
+        return self.media.playlist_name
+
     @property  # type: ignore[misc]
     def media_artist(self) -> str | None:
         """Artist of current playing media, music track only."""
@@ -385,11 +372,6 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
         """Name of the current input source."""
         return self.media.source_name or None
 
-    @property  # type: ignore[misc]
-    def supported_features(self) -> int:
-        """Flag media player features that are supported."""
-        return SUPPORT_SONOS
-
     @soco_error()
     def volume_up(self) -> None:
         """Volume up media player."""
@@ -508,6 +490,9 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
 
         If media_id is a Plex payload, attempt Plex->Sonos playback.
 
+        If media_id is a Sonos or Tidal share link, attempt playback
+        using the respective service.
+
         If media_type is "playlist", media_id should be a Sonos
         Playlist name.  Otherwise, media_id should be a URI.
 
@@ -517,27 +502,21 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
         if media_id and media_id.startswith(PLEX_URI_SCHEME):
             media_id = media_id[len(PLEX_URI_SCHEME) :]
             play_on_sonos(self.hass, media_type, media_id, self.name)  # type: ignore[no-untyped-call]
+            return
+
+        share_link = self.speaker.share_link
+        if share_link.is_share_link(media_id):
+            if kwargs.get(ATTR_MEDIA_ENQUEUE):
+                share_link.add_share_link_to_queue(media_id)
+            else:
+                soco.clear_queue()
+                share_link.add_share_link_to_queue(media_id)
+                soco.play_from_queue(0)
         elif media_type in (MEDIA_TYPE_MUSIC, MEDIA_TYPE_TRACK):
             if kwargs.get(ATTR_MEDIA_ENQUEUE):
-                try:
-                    if soco.is_service_uri(media_id):
-                        soco.add_service_uri_to_queue(media_id)
-                    else:
-                        soco.add_uri_to_queue(media_id)
-                except SoCoUPnPException:
-                    _LOGGER.error(
-                        'Error parsing media uri "%s", '
-                        "please check it's a valid media resource "
-                        "supported by Sonos",
-                        media_id,
-                    )
+                soco.add_uri_to_queue(media_id)
             else:
-                if soco.is_service_uri(media_id):
-                    soco.clear_queue()
-                    soco.add_service_uri_to_queue(media_id)
-                    soco.play_from_queue(0)
-                else:
-                    soco.play_uri(media_id)
+                soco.play_uri(media_id)
         elif media_type == MEDIA_TYPE_PLAYLIST:
             if media_id.startswith("S:"):
                 item = get_media(self.media.library, media_id, media_type)  # type: ignore[no-untyped-call]
@@ -546,11 +525,12 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
             try:
                 playlists = soco.get_sonos_playlists()
                 playlist = next(p for p in playlists if p.title == media_id)
+            except StopIteration:
+                _LOGGER.error('Could not find a Sonos playlist named "%s"', media_id)
+            else:
                 soco.clear_queue()
                 soco.add_to_queue(playlist)
                 soco.play_from_queue(0)
-            except StopIteration:
-                _LOGGER.error('Could not find a Sonos playlist named "%s"', media_id)
         elif media_type in PLAYABLE_MEDIA_TYPES:
             item = get_media(self.media.library, media_id, media_type)  # type: ignore[no-untyped-call]
 
@@ -599,27 +579,6 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
             alarm.include_linked_zones = include_linked_zones
         alarm.save()
 
-    @soco_error()
-    def set_option(
-        self,
-        buttons_enabled: bool | None = None,
-        night_sound: bool | None = None,
-        speech_enhance: bool | None = None,
-        status_light: bool | None = None,
-    ) -> None:
-        """Modify playback options."""
-        if buttons_enabled is not None:
-            self.soco.buttons_enabled = buttons_enabled
-
-        if night_sound is not None and self.speaker.night_mode is not None:
-            self.soco.night_mode = night_sound
-
-        if speech_enhance is not None and self.speaker.dialog_mode is not None:
-            self.soco.dialog_mode = speech_enhance
-
-        if status_light is not None:
-            self.soco.status_light = status_light
-
     @soco_error()
     def play_queue(self, queue_position: int = 0) -> None:
         """Start playing the queue."""
@@ -637,12 +596,6 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
             ATTR_SONOS_GROUP: self.speaker.sonos_group_entities
         }
 
-        if self.speaker.night_mode is not None:
-            attributes[ATTR_NIGHT_SOUND] = self.speaker.night_mode
-
-        if self.speaker.dialog_mode is not None:
-            attributes[ATTR_SPEECH_ENHANCE] = self.speaker.dialog_mode
-
         if self.media.queue_position is not None:
             attributes[ATTR_QUEUE_POSITION] = self.media.queue_position
 
@@ -665,8 +618,7 @@ class SonosMediaPlayerEntity(SonosEntity, MediaPlayerEntity):
                 media_content_id,
                 MEDIA_TYPES_TO_SONOS[media_content_type],
             )
-            image_url = getattr(item, "album_art_uri", None)
-            if image_url:
+            if image_url := getattr(item, "album_art_uri", None):
                 result = await self._async_fetch_image(image_url)  # type: ignore[no-untyped-call]
                 return result  # type: ignore
 
