@@ -12,6 +12,7 @@ from homeassistant import config_entries
 from homeassistant.const import (
     ATTR_ENTITY_ID,
     ATTR_NAME,
+    ATTR_VIA_DEVICE,
     EVENT_HOMEASSISTANT_START,
     EVENT_HOMEASSISTANT_STOP,
 )
@@ -25,11 +26,10 @@ from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
 )
-from homeassistant.helpers.entity import generate_entity_id
+from homeassistant.helpers.entity import DeviceInfo, generate_entity_id
 from homeassistant.helpers.entity_component import DEFAULT_SCAN_INTERVAL
 from homeassistant.helpers.entity_platform import EntityPlatform
 from homeassistant.helpers.entity_registry import (
-    async_entries_for_config_entry,
     async_get_registry as async_get_entity_registry,
 )
 from homeassistant.helpers.entity_values import EntityValues
@@ -56,11 +56,18 @@ from .const import (
     DOMAIN,
 )
 from .discovery_schemas import DISCOVERY_SCHEMAS
+from .migration import (  # noqa: F401
+    async_add_migration_entity_value,
+    async_get_migration_data,
+    async_is_ozw_migrated,
+    async_is_zwave_js_migrated,
+)
 from .node_entity import ZWaveBaseEntity, ZWaveNodeEntity
 from .util import (
     check_has_unique_id,
     check_node_schema,
     check_value_schema,
+    compute_value_unique_id,
     is_node_parsed,
     node_device_id_and_name,
     node_name,
@@ -253,64 +260,6 @@ CONFIG_SCHEMA = vol.Schema(
 )
 
 
-async def async_get_ozw_migration_data(hass):
-    """Return dict with info for migration to ozw integration."""
-    data_to_migrate = {}
-
-    zwave_config_entries = hass.config_entries.async_entries(DOMAIN)
-    if not zwave_config_entries:
-        _LOGGER.error("Config entry not set up")
-        return data_to_migrate
-
-    if hass.data.get(DATA_ZWAVE_CONFIG_YAML_PRESENT):
-        _LOGGER.warning(
-            "Remove %s from configuration.yaml "
-            "to avoid setting up this integration on restart "
-            "after completing migration to ozw",
-            DOMAIN,
-        )
-
-    config_entry = zwave_config_entries[0]  # zwave only has a single config entry
-    ent_reg = await async_get_entity_registry(hass)
-    entity_entries = async_entries_for_config_entry(ent_reg, config_entry.entry_id)
-    unique_entries = {entry.unique_id: entry for entry in entity_entries}
-    dev_reg = await async_get_device_registry(hass)
-
-    for entity_values in hass.data[DATA_ENTITY_VALUES]:
-        node = entity_values.primary.node
-        unique_id = compute_value_unique_id(node, entity_values.primary)
-        if unique_id not in unique_entries:
-            continue
-        device_identifier, _ = node_device_id_and_name(
-            node, entity_values.primary.instance
-        )
-        device_entry = dev_reg.async_get_device({device_identifier}, set())
-        data_to_migrate[unique_id] = {
-            "node_id": node.node_id,
-            "node_instance": entity_values.primary.instance,
-            "device_id": device_entry.id,
-            "command_class": entity_values.primary.command_class,
-            "command_class_label": entity_values.primary.label,
-            "value_index": entity_values.primary.index,
-            "unique_id": unique_id,
-            "entity_entry": unique_entries[unique_id],
-        }
-
-    return data_to_migrate
-
-
-@callback
-def async_is_ozw_migrated(hass):
-    """Return True if migration to ozw is done."""
-    ozw_config_entries = hass.config_entries.async_entries("ozw")
-    if not ozw_config_entries:
-        return False
-
-    ozw_config_entry = ozw_config_entries[0]  # only one ozw entry is allowed
-    migrated = bool(ozw_config_entry.data.get("migrated"))
-    return migrated
-
-
 def _obj_to_dict(obj):
     """Convert an object into a hash for debug."""
     return {
@@ -404,9 +353,22 @@ async def async_setup_entry(hass, config_entry):  # noqa: C901
     # pylint: enable=import-error
     from pydispatch import dispatcher
 
-    if async_is_ozw_migrated(hass):
+    if async_is_ozw_migrated(hass) or async_is_zwave_js_migrated(hass):
+
+        if hass.data.get(DATA_ZWAVE_CONFIG_YAML_PRESENT):
+            config_yaml_message = (
+                ", and remove %s from configuration.yaml "
+                "to avoid setting up this integration on restart ",
+                DOMAIN,
+            )
+        else:
+            config_yaml_message = ""
+
         _LOGGER.error(
-            "Migration to ozw has been done. Please remove the zwave integration"
+            "Migration away from legacy Z-Wave has been done. "
+            "Please remove the %s integration%s",
+            DOMAIN,
+            config_yaml_message,
         )
         return False
 
@@ -528,7 +490,7 @@ async def async_setup_entry(hass, config_entry):  # noqa: C901
             await platform.async_add_entities([entity])
 
         if entity.unique_id:
-            hass.async_add_job(_add_node_to_component())
+            hass.create_task(_add_node_to_component())
             return
 
         @callback
@@ -1114,8 +1076,8 @@ class ZWaveDeviceEntityValues:
         """
         if not check_node_schema(value.node, self._schema):
             return
-        for name in self._values:
-            if self._values[name] is not None:
+        for name, name_value in self._values.items():
+            if name_value is not None:
                 continue
             if not check_value_schema(value, self._schema[const.DISC_VALUES][name]):
                 continue
@@ -1307,6 +1269,9 @@ class ZWaveDeviceEntity(ZWaveBaseEntity):
             self.refresh_from_network,
         )
 
+        # Add legacy Z-Wave migration data.
+        await async_add_migration_entity_value(self.hass, self.entity_id, self.values)
+
     def _update_attributes(self):
         """Update the node attributes. May only be used inside callback."""
         self.node_id = self.node.node_id
@@ -1337,21 +1302,21 @@ class ZWaveDeviceEntity(ZWaveBaseEntity):
         return self._unique_id
 
     @property
-    def device_info(self):
+    def device_info(self) -> DeviceInfo:
         """Return device information."""
         identifier, name = node_device_id_and_name(
             self.node, self.values.primary.instance
         )
-        info = {
-            "name": name,
-            "identifiers": {identifier},
-            "manufacturer": self.node.manufacturer_name,
-            "model": self.node.product_name,
-        }
+        info = DeviceInfo(
+            name=name,
+            identifiers={identifier},
+            manufacturer=self.node.manufacturer_name,
+            model=self.node.product_name,
+        )
         if self.values.primary.instance > 1:
-            info["via_device"] = (DOMAIN, self.node_id)
+            info[ATTR_VIA_DEVICE] = (DOMAIN, self.node_id)
         elif self.node_id > 1:
-            info["via_device"] = (DOMAIN, 1)
+            info[ATTR_VIA_DEVICE] = (DOMAIN, 1)
         return info
 
     @property
@@ -1386,8 +1351,3 @@ class ZWaveDeviceEntity(ZWaveBaseEntity):
         ) or self.node.is_ready:
             return compute_value_unique_id(self.node, self.values.primary)
         return None
-
-
-def compute_value_unique_id(node, value):
-    """Compute unique_id a value would get if it were to get one."""
-    return f"{node.node_id}-{value.object_id}"
