@@ -2,11 +2,11 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Awaitable, Iterable
+from collections.abc import Awaitable, Callable, Iterable
 import dataclasses
 from functools import partial, wraps
 import logging
-from typing import TYPE_CHECKING, Any, Callable, TypedDict
+from typing import TYPE_CHECKING, Any, TypedDict
 
 import voluptuous as vol
 
@@ -20,6 +20,7 @@ from homeassistant.const import (
     CONF_SERVICE_DATA,
     CONF_SERVICE_TEMPLATE,
     CONF_TARGET,
+    ENTITY_CATEGORIES,
     ENTITY_MATCH_ALL,
     ENTITY_MATCH_NONE,
 )
@@ -227,12 +228,17 @@ def async_prepare_call_from_config(
 
     service_data = {}
 
-    for conf in [CONF_SERVICE_DATA, CONF_SERVICE_DATA_TEMPLATE]:
+    for conf in (CONF_SERVICE_DATA, CONF_SERVICE_DATA_TEMPLATE):
         if conf not in config:
             continue
         try:
             template.attach(hass, config[conf])
-            service_data.update(template.render_complex(config[conf], variables))
+            render = template.render_complex(config[conf], variables)
+            if not isinstance(render, dict):
+                raise HomeAssistantError(
+                    "Error rendering data template: Result is not a Dictionary"
+                )
+            service_data.update(render)
         except TemplateError as ex:
             raise HomeAssistantError(f"Error rendering data template: {ex}") from ex
 
@@ -279,9 +285,7 @@ async def async_extract_entities(
     if data_ent_id == ENTITY_MATCH_ALL:
         return [entity for entity in entities if entity.available]
 
-    referenced = await async_extract_referenced_entity_ids(
-        hass, service_call, expand_group
-    )
+    referenced = async_extract_referenced_entity_ids(hass, service_call, expand_group)
     combined = referenced.referenced | referenced.indirectly_referenced
 
     found = []
@@ -310,9 +314,7 @@ async def async_extract_entity_ids(
 
     Will convert group entity ids to the entity ids it represents.
     """
-    referenced = await async_extract_referenced_entity_ids(
-        hass, service_call, expand_group
-    )
+    referenced = async_extract_referenced_entity_ids(hass, service_call, expand_group)
     return referenced.referenced | referenced.indirectly_referenced
 
 
@@ -322,7 +324,7 @@ def _has_match(ids: str | list | None) -> bool:
 
 
 @bind_hass
-async def async_extract_referenced_entity_ids(
+def async_extract_referenced_entity_ids(
     hass: HomeAssistant, service_call: ServiceCall, expand_group: bool = True
 ) -> SelectedEntities:
     """Extract referenced entity IDs from a service call."""
@@ -363,6 +365,10 @@ async def async_extract_referenced_entity_ids(
         return selected
 
     for ent_entry in ent_reg.entities.values():
+        # Do not add config or diagnostic entities referenced by areas or devices
+        if ent_entry.entity_category in ENTITY_CATEGORIES:
+            continue
+
         if (
             # when area matches the target area
             ent_entry.area_id in selector.area_ids
@@ -384,19 +390,18 @@ async def async_extract_config_entry_ids(
     hass: HomeAssistant, service_call: ServiceCall, expand_group: bool = True
 ) -> set:
     """Extract referenced config entry ids from a service call."""
-    referenced = await async_extract_referenced_entity_ids(
-        hass, service_call, expand_group
-    )
+    referenced = async_extract_referenced_entity_ids(hass, service_call, expand_group)
     ent_reg = entity_registry.async_get(hass)
     dev_reg = device_registry.async_get(hass)
     config_entry_ids: set[str] = set()
 
     # Some devices may have no entities
     for device_id in referenced.referenced_devices:
-        if device_id in dev_reg.devices:
-            device = dev_reg.async_get(device_id)
-            if device is not None:
-                config_entry_ids.update(device.config_entries)
+        if (
+            device_id in dev_reg.devices
+            and (device := dev_reg.async_get(device_id)) is not None
+        ):
+            config_entry_ids.update(device.config_entries)
 
     for entity_id in referenced.referenced | referenced.indirectly_referenced:
         entry = ent_reg.async_get(entity_id)
@@ -545,7 +550,7 @@ async def entity_service_call(
         all_referenced: set[str] | None = None
     else:
         # A set of entities we're trying to target.
-        referenced = await async_extract_referenced_entity_ids(hass, call, True)
+        referenced = async_extract_referenced_entity_ids(hass, call, True)
         all_referenced = referenced.referenced | referenced.indirectly_referenced
 
     # If the service function is a string, we'll pass it the service call data
